/*
    tsync - a utility to sync remote encrypted copies of a tstore
    Copyright (C) 2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tsync-client.c,v 1.13 2007/01/09 21:45:17 kasperd Exp $
*/

#include "tsync.h"
#include <sys/wait.h>

int main(int argc, char ** argv)
{
  char header_buffer[512];
  char buffer[3000];
  off_t seek_location;
  uint32_t record_count;
  int storage_fd,server_rfd,server_wfd,gpg_rfd,gpg_wfd;
  struct stat stbuf;
  pid_t feeder_pid;
  int r;

  char *cookie =getenv("TSYNC_COOKIE");
  char *pass   =getenv("TSYNC_PASS");
  char *storage=getenv("TSYNC_STORAGE");

  if (argc < 2) {
    fprintf(stderr,"Usage: %s <cmd>\n",argv[0]);
    return EXIT_FAILURE;
  }

  if (!(cookie&&pass&&storage)) {
    fprintf(stderr,"TSYNC_ variables missing in environment\n");
    return EXIT_FAILURE;
  }

  storage_fd=open(storage,O_RDONLY);
  if (storage_fd==-1) {
    perror(storage);
    return EXIT_FAILURE;
  }
  fcntl(storage_fd,F_SETFD,1); /* close-on-exec */

  if (fstat(storage_fd,&stbuf)) {
    perror("fstat");
    return EXIT_FAILURE;
  }

  start_server(&server_rfd,&server_wfd,argv+1,&record_count);
  //printf("%u\n",record_count);
  switch(record_count) {
  case 0:
    fprintf(stderr,"No records in encrypted storage, there should at least be an encrypted password\n");
    return EXIT_FAILURE;
  case 1:
    seek_location=512;
    break;
  default:
    {
      uint32_t vv=htonl(record_count-1);
      assert(write(server_wfd,"S",1)==1);
      assert(write(server_wfd,&vv,4)==4);
      vv=htonl(8);
      assert(write(server_wfd,"R",1)==1);
      assert(write(server_wfd,&vv,4)==4);
      assert(read(server_rfd,&vv,4)==4);
      assert(ntohl(vv)==8);
      assert(read(server_rfd,buffer,8)==8);
      seek_location=ntoh64((uint32_t*)buffer);
    }
  }

  if (stbuf.st_size <= seek_location) {
    if (stbuf.st_size == seek_location) {
      fprintf(stderr,"No new data to sync\n");
    } else {
      fprintf(stderr,"Local storage shorter than expected\n");
    }
    return EXIT_FAILURE;
  }
  printf("Syncing %lld new bytes\n",stbuf.st_size-seek_location);
  fflush(stdout);
  assert(stbuf.st_size>seek_location);

  start_gpg(&gpg_rfd,&gpg_wfd,pass,1);
  assert(read(storage_fd,header_buffer,512)==512);
  assert(write(gpg_wfd,header_buffer,512)==512);
  lseek(storage_fd,seek_location,SEEK_SET);

  switch(feeder_pid=fork()) {
  case -1:
    perror("fork");
    return EXIT_FAILURE;
  case 0:
    close(server_rfd);
    close(server_wfd);
    close(gpg_rfd);
    while(seek_location < stbuf.st_size) {
      int r;
      uint64_t maxtrans = stbuf.st_size-seek_location;
      if (maxtrans>3000) maxtrans=3000;
      r=read(storage_fd,buffer,maxtrans);
      if (r<1) {
	perror(storage);
	_exit(EXIT_FAILURE);
      }
      assert(write(gpg_wfd,buffer,r)==r);
      seek_location+=r;
    }
    close(gpg_wfd);
    _exit(0);
  }
  close(gpg_wfd);
  
  {
    uint32_t l=htonl(8);
    assert(write(server_wfd,"LW",2)==2);
    assert(write(server_wfd,&l,4)==4);
    assert(write(server_wfd,hton64(stbuf.st_size),8)==8);
  }

  while((r=read(gpg_rfd,buffer,3000))>0) {
    uint32_t l=htonl(r);
    assert(write(server_wfd,"W",1)==1);
    assert(write(server_wfd,&l,4)==4);
    assert(write(server_wfd,buffer,r)==r);
  }

  if (r<0) {
    perror("read");
    return EXIT_FAILURE;
  }

  assert(waitpid(feeder_pid,&r,0)==feeder_pid);
  assert(WIFEXITED(r)&&(WEXITSTATUS(r)==0)&&(!WIFSIGNALED(r)));

  assert(write(server_wfd,"C",1)==1);
  assert(read(server_rfd,buffer,1)==1);
  assert(buffer[0]=='C');

  return 0;
}
