/*
    tsync - a utility to sync remote encrypted copies of a tstore
    Copyright (C) 2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tsync-ls.c,v 1.3 2006/03/03 08:29:56 kasperd Exp $
*/

#include "tsync.h"

uint32_t header[16];
#define HEADER ((uint8_t*)(header))
#define EXPECTED_SIZE (64+(ntoh64(header+14)<<3))

int storage_fd;

int verify_size()
{
  struct stat stbuf;
  if (fstat(storage_fd,&stbuf)) return -1;
  return (stbuf.st_size<EXPECTED_SIZE);}

int read_header()
{
  switch (read(storage_fd,HEADER,64)) {
  case 64: return (strncmp(HEADER,"TSYNC-1",8)||
		   HEADER[7]||HEADER[51])?1:verify_size();
  case -1: return -1;
  default: return 1;
  }
}

struct record_info {
  uint64_t record_len,storage_len;
} *ri;

int main(int argc, char ** argv)
{
  uint32_t total_records;
  uint64_t start_of_record,end_of_record,record_len;
  uint32_t i;

  long long raw_len,encrypted_len;

  if (argc != 2) {
    fprintf(stderr,"Usage: %s <encrypted-storage>\n",argv[0]);
    return EXIT_FAILURE;
  }

  storage_fd=open(argv[1],O_RDONLY);
  if (storage_fd==-1) {
    perror(argv[1]);
    return EXIT_FAILURE;
  }

  switch(read_header()) {
  case 1:
    fprintf(stderr,"%s: \"%s\" has invalid format\n",argv[0],argv[1]);
    return EXIT_FAILURE;
  case 0:
    break;
  default:
    perror(argv[1]);
    return EXIT_FAILURE;
  }

  total_records=ntohl(*(uint32_t*)(HEADER+52));
  printf("Cookie: %s\n",HEADER+8);
  printf("Records: %d\n",total_records);

  if (!total_records) return EXIT_SUCCESS;

  assert(ri=malloc(sizeof(struct record_info)*total_records));

  start_of_record=EXPECTED_SIZE;

  for (i=total_records;i--;) {
    uint32_t buf[2];
    end_of_record=start_of_record-8;
    switch(pread(storage_fd,buf,8,end_of_record)){
    case -1:
      perror("pread");
      return EXIT_FAILURE;
    case 8:
      break;
    default:
      fprintf(stderr,"Short pread on storage\n");
      exit(EXIT_FAILURE);
    }
    ri[i].record_len=record_len=ntoh64(buf);
    start_of_record=end_of_record-((record_len+7)&~(uint64_t)7);

    switch(pread(storage_fd,buf,8,start_of_record)){
    case -1:
      perror("pread");
      return EXIT_FAILURE;
    case 8:
      break;
    default:
      fprintf(stderr,"Short pread on storage\n");
      exit(EXIT_FAILURE);
    }
    ri[i].storage_len=ntoh64(buf);
  }

  for (i=0;i<total_records;++i) {
    encrypted_len=ri[i].record_len;
    switch(i) {
    case 0:
      raw_len=43;
      break;
    case 1:
      raw_len=ri[i].storage_len;
      encrypted_len-=8;
      break;
    default:
      raw_len=(ri[i].storage_len-ri[i-1].storage_len)+512;
      encrypted_len-=8;
      break;
    }

    printf("%d: %lld => %lld (%.1f%%)\n",i,raw_len,encrypted_len,
	   (100.0*(double)encrypted_len)/(double)raw_len);
  }

  uint32_t buf[2];
  switch(pread(storage_fd,buf,8,EXPECTED_SIZE)){
  case 0:
    return EXIT_SUCCESS;
  case -1:
    perror("pread");
    return EXIT_FAILURE;
  case 8:
    break;
  default:
    fprintf(stderr,"Short pread on storage\n");
    exit(EXIT_FAILURE);
  }

  raw_len=ntoh64(buf);
  if (total_records>1) raw_len-=(ri[total_records-1].storage_len-512);

  printf("%d: %lld => ...\n",total_records,raw_len);

  return EXIT_SUCCESS;
}
